@@ -6,7 +6,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2020	PgPool Global Development Group
+ * Copyright (c) 2003-2018	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -83,7 +83,7 @@ pool_init_pool_passwd(char *pool_passwd_filename, POOL_PASSWD_MODE mode)
 		}
 		ereport(ERROR,
 				(errmsg("initializing pool password, failed to open file:\"%s\"", pool_passwd_filename),
-				 errdetail("file open failed with error:\"%m\"")));
+				 errdetail("file open failed with error:\"%s\"", strerror(errno))));
 	}
 }
 
@@ -451,10 +451,10 @@ pool_reopen_passwd_file(void)
 }
 
 /*
- * function first uses the password in the argument, if the
- * argument is empty string or NULL, it looks for the password
- * for uset in pool_passwd file.
- * The returned password is always in plain text and palloc'd (if not null)
+ * function first uses the password in the argument, if the argument is empty
+ * string or NULL, it looks for the password for uset in pool_passwd file.
+ * The returned password is always in plain text or md5 hashed and palloc'd
+ * (if not null)
  */
 char *
 get_pgpool_config_user_password(char *username, char *password_in_config)
@@ -464,7 +464,7 @@ get_pgpool_config_user_password(char *username, char *password_in_config)
 	PasswordMapping *password_mapping = NULL;
 
 	/*
-	 * if the password specified in config is empty strin or NULL look for the
+	 * if the password specified in confg is empty strin or NULL look for the
 	 * password in pool_passwd file
 	 */
 	if (password_in_config == NULL || strlen(password_in_config) == 0)
@@ -516,11 +516,13 @@ get_pgpool_config_user_password(char *username, char *password_in_config)
 			password = (char*)(password + strlen(PASSWORD_TEXT_PREFIX));
 	}
 
-	if (password && strlen(password) && passwordType != PASSWORD_TYPE_PLAINTEXT)
+	if (password && strlen(password) && (passwordType != PASSWORD_TYPE_PLAINTEXT &&
+										 passwordType != PASSWORD_TYPE_MD5))
 	{
 		/*
-		 * Could not find plain text password in pool_passwd corresponding to the user.
-		 * This is normal. Just use empty password in pgpool.conf.
+		 * Could not find either plain text or md5 hashed password in
+		 * pool_passwd corresponding to the user.  This is normal. Just use
+		 * empty password in pgpool.conf.
 		 */
 		ereport(DEBUG5,
 				(errmsg("could not get the password for user:%s", username),
@@ -619,28 +621,14 @@ read_pool_key(char *key_file_path)
 	if (strlen(key_file_path) == 0)
 		return NULL;
 
-	/*
-	 * To prevent file-swapping due to file race conditions,
-	 * we open the key file before checking it by stat().
-	 */
 	/* If password file cannot be opened, ignore it. */
-	if ( (fp = fopen(key_file_path, "r")) == NULL)
+	if (stat(key_file_path, &stat_buf) != 0)
 		return NULL;
 
-	if (fstat(fileno(fp), &stat_buf) != 0)
-	{
-		ereport(WARNING,
-				(errmsg("failed to stat pool key file"),
-				 errdetail("fstat failed with reason: \"%m\"")));
-		fclose(fp);
-		return NULL;
-	}
-
 	if (!S_ISREG(stat_buf.st_mode))
 	{
 		ereport(WARNING,
 				(errmsg("pool key file \"%s\" is not a plain file\n", key_file_path)));
-		fclose(fp);
 		return NULL;
 	}
 
@@ -651,10 +639,13 @@ read_pool_key(char *key_file_path)
 				(errmsg("pool key file \"%s\" has group or world access; permissions should be u=rw (0600) or less\n",
 						key_file_path)));
 		/* do we want to allow unsecure pool key file ? */
-		/* fclose(fp); */
 		/* return NULL; */
 	}
 
+	fp = fopen(key_file_path, "r");
+	if (fp == NULL)
+		return NULL;
+
 	while (!feof(fp) && !ferror(fp))
 	{
 		int			len;
