@@ -8,8 +8,8 @@
  * Definitions that are needed outside the core parser should be in parser.h.
  *
  *
- * Portions Copyright (c) 2003-2017, PgPool Global Development Group
- * Portions Copyright (c) 1996-2017, PostgreSQL Global Development Group
+ * Portions Copyright (c) 2003-2019, PgPool Global Development Group
+ * Portions Copyright (c) 1996-2019, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * src/include/parser/gramparse.h
@@ -23,7 +23,8 @@
 #include "parsenodes.h"
 #include "scanner.h"
 
-#define AMTYPE_INDEX                    'i'     /* index access method */
+#define AMTYPE_INDEX                    'i' /* index access method */
+#define AMTYPE_TABLE                    't' /* table access method */
 
 /*
  * NB: include gram.h only AFTER including scanner.h, because scanner.h
@@ -68,11 +69,17 @@ typedef struct base_yy_extra_type
 
 
 /* from parser.c */
-extern int base_yylex(YYSTYPE *lvalp, YYLTYPE *llocp,
-		   core_yyscan_t yyscanner);
+extern int	base_yylex(YYSTYPE *lvalp, YYLTYPE *llocp,
+					   core_yyscan_t yyscanner);
+extern int	minimal_base_yylex(YYSTYPE *lvalp, YYLTYPE *llocp,
+					   core_yyscan_t yyscanner);
 
 /* from gram.y */
 extern void parser_init(base_yy_extra_type *yyext);
 extern int	base_yyparse(core_yyscan_t yyscanner);
 
+/* from gram_minimal.y */
+extern void minimal_parser_init(base_yy_extra_type *yyext);
+extern int	minimal_base_yyparse(core_yyscan_t yyscanner);
+
 #endif							/* GRAMPARSE_H */
